/* This code shows how MDMA bandwidth limit feature can be used to limit the MDMA
 * throughput to maximum value.
 */

#include "main.h"
#include <sys/platform.h>
#include <sys/adi_core.h>

CGU_Settings CGU0_PARAM_LIST_DEFAULT;
DMC_PARAM DMC_Param_List;
MDMA_PARAM_LIST MDMAParam;

int main(void)
{
	int failure=0; int i; float fThroughput;

	//Initialize the DMC parameters list
	DMC_Param_List.dmc_no=0;
	DMC_Param_List.ddr_mode=DDR3_MODE;
	DMC_Param_List.phy_init_required=true;
	DMC_Param_List.cclk_dclk_ratio=1;
	DMC_Param_List.anomaly_20000037_applicable=true;
	DMC_Param_List.dmc_ctl_value=DMC_CTL_VALUE;
	DMC_Param_List.dmc_cfg_value=DMC_CFG_VALUE;
	DMC_Param_List.dmc_tr0_value=DMC_TR0_VALUE;
	DMC_Param_List.dmc_tr1_value=DMC_TR1_VALUE;
	DMC_Param_List.dmc_tr2_value=DMC_TR2_VALUE;
	DMC_Param_List.dmc_mr0_value=DMC_MR0_VALUE;
	DMC_Param_List.dmc_mr1_value=DMC_MR1_VALUE;
	DMC_Param_List.dmc_mr2_value=DMC_MR2_VALUE;
	DMC_Param_List.padctl2_value=DMC_PADCTL2_VALUE;
	DMC_Param_List.dmc_cphyctl_value=DMC_CPHYCTL_VALUE;
	DMC_Param_List.dmc_data_calib_add_value=DMC0_DATA_CALIB_ADD;

	//Initialize CGU
	CGUInit();

	//Initialize DMC0
	DMCInit(DMC_Param_List);

	DMC_Param_List.dmc_no=1;
	DMC_Param_List.dmc_data_calib_add_value=DMC1_DATA_CALIB_ADD;

	//Initialize DMC1
	DMCInit(DMC_Param_List);

	//Configuring sync mode
	*pREG_SCB1_MST00_SYNC=2;

	MDMAParam.uiMDMAStreamNo=3;
	MDMAParam.fMDMAClockSpeedMHz=225;
	MDMAParam.uiBytes=16384;
	MDMAParam.uiSrcMem=MEMTYPE_L3_DMC0;
	MDMAParam.uiDestMem=MEMTYPE_L1S1;
	MDMAParam.uiSrcMSIZE=8;
	MDMAParam.uiDstMSIZE=32;

	//Configure bandwidth limit register here (BWLCNT = SSYSCLK speed in MHz*MSIZE in bytes/Bandwidth required in MB/s)
	*pREG_DMA43_BWLCNT=128;

	fThroughput=MDMA_Throughput_Measure(MDMAParam);
	printf("\nNo. of Bytes=%d\tThroughput=%0.2f MB/s",MDMAParam.uiBytes,fThroughput);

	return 0;
}

void CGUInit(void)
{
	int i;

	//PLLCLK = 450 MHz, CCLK = 450 MHz, SYSCLK = 225 MHz, DCLK = 450 MHz, SCLK0 = SCLK1 = 225 MHz
	CGU0_PARAM_LIST_DEFAULT.ctl_MSEL=18;
	CGU0_PARAM_LIST_DEFAULT.ctl_DF=0;
	CGU0_PARAM_LIST_DEFAULT.div_CSEL=1;
	CGU0_PARAM_LIST_DEFAULT.div_SYSSEL=2;
	CGU0_PARAM_LIST_DEFAULT.div_S0SEL=2;
	CGU0_PARAM_LIST_DEFAULT.div_S1SEL=2;
	CGU0_PARAM_LIST_DEFAULT.div_DSEL=1;

	//Set DMCx_PHY_CTL0.RESETDLL bit.
	*pREG_DMC0_PHY_CTL0=0x800;
	//*pREG_DMC1_PHY_CTL0=0x800;

	//Initialize CGU to change the DCLK frequency
	CGU_Init(0, 0, &CGU0_PARAM_LIST_DEFAULT,  0);

	//Clear DMCx_PHY_CTL0.RESETDLL bit
	*pREG_DMC0_PHY_CTL0=0x000;
	//*pREG_DMC1_PHY_CTL0=0x000;

	//Wait for 9000 DCLK cycles for the DLL to lock
	for(i=0;i<9000*DMC_Param_List.cclk_dclk_ratio;i++);
}
