/*This code shows how bandwidth monitor feature of MDMA can be used to measure/monitor
 * MDMA bandwidth
 */

#include "main.h"
#include <sys/platform.h>
#include <sys/adi_core.h>
#include "time.h"

CGU_Settings CGU0_PARAM_LIST_DEFAULT;
DMC_PARAM DMC_Param_List;

void MDMA3_Interrupt_Handler(uint32_t iid, void* handlerArg);

#pragma section("seg_l1_block1")
uint32_t ucIntBuff1[16384];

#pragma section("seg_l1_block2")
uint32_t ucIntBuff2[16384];

//Global variables
volatile clock_t clock_start;
volatile clock_t clock_stop_mdma3;
volatile clock_t cycles_mdma3;
float throughput_mdma3;
int DMA_Done_MDMA3=0;
int BWMCNT_Expired=0;
volatile int bwmcount_start=4096; //Min BW of 900 MB/s
volatile int bwmcount_stop;
volatile int bm_cycles;
float bm_throughput;

int main(void)
{
	int failure=0; int i; float fThroughput;

	//Initialize the DMC parameters list
	DMC_Param_List.dmc_no=0;
	DMC_Param_List.ddr_mode=DDR3_MODE;
	DMC_Param_List.phy_init_required=true;
	DMC_Param_List.cclk_dclk_ratio=1;
	DMC_Param_List.anomaly_20000037_applicable=true;
	DMC_Param_List.dmc_ctl_value=DMC_CTL_VALUE;
	DMC_Param_List.dmc_cfg_value=DMC_CFG_VALUE;
	DMC_Param_List.dmc_tr0_value=DMC_TR0_VALUE;
	DMC_Param_List.dmc_tr1_value=DMC_TR1_VALUE;
	DMC_Param_List.dmc_tr2_value=DMC_TR2_VALUE;
	DMC_Param_List.dmc_mr0_value=DMC_MR0_VALUE;
	DMC_Param_List.dmc_mr1_value=DMC_MR1_VALUE;
	DMC_Param_List.dmc_mr2_value=DMC_MR2_VALUE;
	DMC_Param_List.padctl2_value=DMC_PADCTL2_VALUE;
	DMC_Param_List.dmc_cphyctl_value=DMC_CPHYCTL_VALUE;
	DMC_Param_List.dmc_data_calib_add_value=DMC0_DATA_CALIB_ADD;

	//Initialize CGU
	CGUInit();

	//Initialize DMC0
	DMCInit(DMC_Param_List);

	DMC_Param_List.dmc_no=1;
	DMC_Param_List.dmc_data_calib_add_value=DMC1_DATA_CALIB_ADD;

	//Initialize DMC1
	DMCInit(DMC_Param_List);

	//Configuring sync mode
	*pREG_SCB1_MST00_SYNC=2;
	*pREG_SCB2_MST00_SYNC=2;

	//DMA buffer size = 16 K bytes
	unsigned int uiBytes=16384;

	//Enable MDMA3 stream as secure master
	*pREG_SPU0_SECUREP141=0x3;

	//Enable MDMA2 stream as secure master
	*pREG_SPU0_SECUREP140=0x3;

	//Initialize interrupts and handlers
	*pREG_SEC0_GCTL=ENUM_SEC_GCTL_EN;
	*pREG_SEC0_CCTL1=ENUM_SEC_CCTL_EN;
	adi_int_InstallHandler(INTR_MDMA3_DST, MDMA3_Interrupt_Handler, NULL, true);

	//Configure MDMA3 (43/44) to read from DMC and
	MDMA_CONFIG(43, SOURCE_DMA_CFG, DMC0_BANK0_START_ADDRESS, SOURCE_MSIZE, uiBytes/SOURCE_MSIZE,44,DEST_DMA_CFG, (uint32_t)ucIntBuff1|0x28000000, DEST_MSIZE, uiBytes/DEST_MSIZE);

	//Initialize the bandwidth monitor register here to a minimum bandwidth of 900 MB/s (=Buffer Size*SCLK speed in MHz/Bandwidth)
	*pREG_DMA44_BWMCNT=bwmcount_start;

	//Enable DMAs now
	*pREG_DMA43_CFG |= 0x1;
	*pREG_DMA44_CFG |= 0x1;

	//Add another MDMA channel here if required
	#ifdef ADD_ANOTHER_MDMA
		 MDMA_CONFIG(39, SOURCE_DMA_CFG1, DMC0_BANK1_START_ADDRESS+16384, SOURCE_MSIZE, uiBytes/SOURCE_MSIZE,40,DEST_DMA_CFG1, (uint32_t)ucIntBuff2|0x28000000, DEST_MSIZE, uiBytes/DEST_MSIZE);
		*pREG_DMA39_CFG |= 0x1;
		*pREG_DMA40_CFG |= 0x1;
	#endif


	//Start the cycle count
	clock_start = clock();

	//Wait till the MDMA0 done
	while(DMA_Done_MDMA3==0)
	{
		//Check if the bandwidth monitor count expired. If yes, print an error message and return
		if(BWMCNT_Expired)
		{
				printf("Bandwidth monitor ERROR !!! BWMCNT_CUR  expired....!!");
				return 0;
		}
	}

	//Calculate the number of cycles required for the MDMA0 to complete
	cycles_mdma3=clock_stop_mdma3-clock_start;
	bm_cycles=bwmcount_start-bwmcount_stop;

	//Disalbe the MDMAs here
	DISABLE_DMAs(18,19);
	DISABLE_DMAs(20,21);

	//Calculate and print the throughput values
	throughput_mdma3=(float)(uiBytes*CCLK_SPEED/cycles_mdma3);
	bm_throughput=(float)(uiBytes*SYSCLK_SPEED/bm_cycles);
	printf("\n\n No of bytes=%d, No of core cycles measured using cycle count=%d\n\n ",uiBytes,cycles_mdma3);
	printf("Throughput measured using cycle count=%f MB/s\n\n",throughput_mdma3, bm_throughput);
	printf("\n\n No of SCLK cycles for using BW monitor=%d\n\n ", bm_cycles);
	printf("Throughput measured using bandwidth monitor=%f MB/s\n\n",throughput_mdma3, bm_throughput);

	return 0;

}

void CGUInit(void)
{
	int i;

	//PLLCLK = 450 MHz, CCLK = 450 MHz, SYSCLK = 225 MHz, DCLK = 450 MHz, SCLK0 = SCLK1 = 225 MHz
	CGU0_PARAM_LIST_DEFAULT.ctl_MSEL=18;
	CGU0_PARAM_LIST_DEFAULT.ctl_DF=0;
	CGU0_PARAM_LIST_DEFAULT.div_CSEL=1;
	CGU0_PARAM_LIST_DEFAULT.div_SYSSEL=2;
	CGU0_PARAM_LIST_DEFAULT.div_S0SEL=2;
	CGU0_PARAM_LIST_DEFAULT.div_S1SEL=2;
	CGU0_PARAM_LIST_DEFAULT.div_DSEL=1;

	//Set DMCx_PHY_CTL0.RESETDLL bit.
	*pREG_DMC0_PHY_CTL0=0x800;
	*pREG_DMC1_PHY_CTL0=0x800;

	//Initialize CGU to change the DCLK frequency
	CGU_Init(0, 0, &CGU0_PARAM_LIST_DEFAULT,  0);

	//Clear DMCx_PHY_CTL0.RESETDLL bit
	*pREG_DMC0_PHY_CTL0=0x000;
	*pREG_DMC1_PHY_CTL0=0x000;

	//Wait for 9000 DCLK cycles for the DLL to lock
	for(i=0;i<9000*DMC_Param_List.cclk_dclk_ratio;i++);
}

void MDMA3_Interrupt_Handler(uint32_t iid, void* handlerArg)
{
	//Stop the cycle count
	clock_stop_mdma3 = clock();

	//Check if the interrupt occurred due to bandwidth monitor counter expiry
	if(*pREG_DMA44_STAT&ENUM_DMA_STAT_BWMONERR)
	{
		*pREG_DMA44_STAT |= ENUM_DMA_STAT_BWMONERR;
		BWMCNT_Expired=1;
	}

	//Check if the interrupt occurred due to the DMA completion
	else if(*pREG_DMA44_STAT&ENUM_DMA_STAT_IRQDONE)
	{
		*pREG_DMA44_STAT |= ENUM_DMA_STAT_IRQDONE;			 //Clear the W1C interrupt status bit
		DMA_Done_MDMA3 = 1;
		bwmcount_stop=*pREG_DMA44_BWMCNT_CUR;
	}
}
